/* eslint-env node */
'use strict';

// Ce modules fournit quelques fonction pour simplifier l'accès
// à notre base de données sqlite

const sqlite3 = require('sqlite3').verbose();
const db = new sqlite3.Database('./PolyMusic.db', sqlite3.OPEN_READWRITE, function (err) {
    if (err) {
        console.error(err + '\n' + 'run "npm run createDB" to create a database file');
        // Pas de problème pour faire un appel synchrone ici : on est dans la phase
        // d'initialisation du serveur et pas dans le traitement de requêtes.
        require('process').exit(-1);
    }
});


// IMPORTANT à savoir. Les fonctions de l'api sqlite3 SYNCHRONES PAR DEFAUT ! 
// Rend la fonction get de l'api sqlite compatible avec les promesses
const get = sql => new Promise(function (resolve, reject) {
    db.get(sql, function (err, row) {
        if (err) {
            reject(err);
        }
        else {
            resolve(row);
        }
    });
});

// Idem pour la fonction all
const all = sql => new Promise(function (resolve, reject) {
    db.all(sql, function (err, rows) {
        if (err) {
            reject(err);
        }
        else {
            resolve(rows);
        }
    });
});

const run = sql => new Promise(function (resolve, reject) {
    db.run(sql, function (err, rows) {
        err ? reject(err) : resolve(rows);
    });
});


// Cet export met à disposition des programmeurs 2 fonctions
// dbhelper.artists.byId, qui récupère les infos d'un artiste particulier (via son id)
// dbhelper.artists.all, qui récupère tous les artistes
module.exports.artists = {
    byId: id => ({
        get albums() {
            return all(`
                select year, album.name
                    from album, artist
                    where artist.id = artist_id
                        and artist.id = ${id | 0};
            `);
        },
    }),
    all: () => all('select id, name from artist'),
};

// Cet export met à disposition des programmeurs 2 fonctions
// utiles pour l'authentification des utilisateurs
// dbhelper.users.byUsername, qui récupère un utilisateur par son nom
// dbhelper.users.byId, qui récupère un utilisateur par son Id
module.exports.users = {
    byUsername: (username) => get(`
        select id, password from user where name = '${username}';
    `),
    a: Promise.resolve({
        id: 0,
        checkPassword: (/*password*/) => true,
    }),
    byId: id => get(`select name as username from user where id = ${id}`),
};








// CRUD FOURNISSEUR from :
module.exports.fournisseur = {
    // READ
    all: () => all('SELECT * FROM FOURNISSEUR;'),

    byNom: (nomFournisseur) => get(`
    SELECT * FROM FOURNISSEUR WHERE nom_fournisseur = '${nomFournisseur}'
    ;`),

    // INSERT INTO FOURNISSEUR VALUES (nf, adr, email, tel, pf);
    create: (nom_fournisseur, adresse, email, telephone) => {

        let sql = `INSERT INTO FOURNISSEUR VALUES ('${nom_fournisseur}', `;

        adresse ? sql += `'${adresse}', ` : sql += `NULL, `;
        email ? sql += `'${email}', ` : sql += `NULL, `;
        telephone ? sql += `${telephone},` : sql += `NULL,`;

        sql = sql.slice(0, -1) + `);`;

        console.log("SQL:", sql);
        return run(sql);
    },

    // UPDATE FOURNISSEUR SET adresse = adr, email = email, telephone = tel, produits_fournis = pf WHERE nom_fournisseur = nf;
    update: (nom_fournisseur_old, nom_fournisseur_new, adresse, email, telephone) => {

        console.log("CHAMPS UPDATES:", nom_fournisseur_old, nom_fournisseur_new, adresse, email, telephone);

        let sql = `UPDATE FOURNISSEUR SET `;

        //si des champs sont à undefined, on ne les modifie pas
        if (nom_fournisseur_new !== undefined) {
            sql += `nom_fournisseur = '${nom_fournisseur_new}', `;
        }
        if (adresse !== undefined) {
            sql += `adresse = '${adresse}', `;
        }
        if (email !== undefined) {
            sql += `email = '${email}', `;
        }
        if (telephone !== undefined) {
            sql += `telephone = ${telephone}, `;
        }

        // note : le slice permet de supprimer la virgule en trop
        sql = sql.slice(0, -2) + ` WHERE nom_fournisseur = '${nom_fournisseur_old}';`;
        console.log("SQL:", sql);

        return run(sql);
    },

    // DELETE 
    delete: (nomFournisseur) =>
        run(`DELETE FROM FOURNISSEUR WHERE nom_fournisseur = '${nomFournisseur}';`),
};



// CRUD PRODUIT from :
// TABLE PRODUIT(
//     nom_produit VARCHAR(50),
//     nom_fournisseur VARCHAR(50),
//     prix_unitaire DOUBLE,
//     quantite_min INT,
//     quantite_max INT,
//     image VARCHAR(512),
//     quantite_course INT,
//     PRIMARY KEY(nom_produit),
//     FOREIGN KEY(nom_fournisseur) REFERENCES FOURNISSEUR(nom_fournisseur)
module.exports.produit = {
    // READ
    all: () => all('SELECT * FROM PRODUIT;'),

    byNom: (nomProduit) => get(`
    SELECT * FROM PRODUIT WHERE nom_produit = '${nomProduit}'
    ;`),

    // INSERT INTO PRODUIT VALUES 
    create: (nom_produit, nom_fournisseur, prix_unitaire, quantite_min, quantite_max, image, quantite_course) => {

        let sql = `INSERT INTO PRODUIT VALUES (`;

        sql += `'${nom_produit}', `;

        nom_fournisseur ? sql += `'${nom_fournisseur}', ` : sql += `NULL, `;
        prix_unitaire ? sql += `${prix_unitaire}, ` : sql += `NULL, `;
        quantite_min ? sql += `${quantite_min}, ` : sql += `NULL, `;
        quantite_max ? sql += `${quantite_max}, ` : sql += `NULL, `;
        image ? sql += `'${image}', ` : sql += `NULL, `;
        quantite_course ? sql += `${quantite_course}, ` : sql += `NULL, `;

        sql = sql.slice(0, -2) + `);`;

        console.log("SQL:", sql);
        return run(sql);
    },

    //update 
    update: (nom_produit_old, nom_produit_new, nom_fournisseur, prix_unitaire, quantite_min, quantite_max, image, quantite_course) => {

        console.log("CHAMPS UPDATES:", nom_produit_old, nom_produit_new, nom_fournisseur, prix_unitaire, quantite_min, quantite_max, image, quantite_course);

        let sql = `UPDATE PRODUIT SET `;

        //si des champs sont à undefined, on ne les modifie pas
        if (nom_produit_new !== undefined) {
            sql += `nom_produit = '${nom_produit_new}', `;
        }
        if (nom_fournisseur !== undefined) {
            sql += `nom_fournisseur = '${nom_fournisseur}', `;
        }
        if (prix_unitaire !== undefined) {
            sql += `prix_unitaire = '${prix_unitaire}', `;
        }
        if (quantite_min !== undefined) {
            sql += `quantite_min = ${quantite_min}, `;
        }
        if (quantite_max !== undefined) {
            sql += `quantite_max = ${quantite_max}, `;
        }
        if (image !== undefined) {
            sql += `image = '${image}', `;
        }
        if (quantite_course !== undefined) {
            sql += `quantite_course = ${quantite_course}, `;
        }

        // note : le slice permet de supprimer la virgule en trop
        sql = sql.slice(0, -2) + ` WHERE nom_produit = '${nom_produit_old}';`;
        console.log("SQL:", sql);

        return run(sql);
    },


    // DELETE
    delete: (nomProduit) =>
        run(`DELETE FROM PRODUIT WHERE nom_produit = '${nomProduit}';`),
};

// CREATE TABLE STOCK(
//     nom_produit VARCHAR(50),
//     date_peremption DATE,
//     quantite_stock INT,
//     PRIMARY KEY(nom_produit, date_peremption),
//     FOREIGN KEY(nom_produit) REFERENCES PRODUIT(nom_produit)
//  );
module.exports.stock = {
    // READ
    all: () => all('SELECT * FROM STOCK;'),

    byNomDate: (nom_produit, date_peremption) => get(`
    SELECT * FROM STOCK WHERE nom_produit = '${nom_produit}' AND date_peremption = '${date_peremption}'
    ;`),

    byNom: (nom_produit) => all(`
    SELECT * FROM STOCK WHERE nom_produit = '${nom_produit}'
    ;`),

    // INSERT INTO STOCK VALUES
    create: (nom_produit, date_peremption, quantite_stock) => {

        let sql = `INSERT INTO STOCK VALUES (`;

        sql += `'${nom_produit}', `;
        sql += `'${date_peremption}', `;
        quantite_stock ? sql += `${quantite_stock}, ` : sql += `0, `;

        sql = sql.slice(0, -2) + `);`;

        console.log("SQL:", sql);
        return run(sql);
    },

    // UPDATE 
    update: (nom_produit_old, date_peremption_old, nom_produit_new, date_peremption_new, quantite_stock) => {

        console.log("CHAMPS UPDATES:", nom_produit_old, date_peremption_old, nom_produit_new, date_peremption_new, quantite_stock);

        let sql = `UPDATE STOCK SET `;

        //si des champs sont à undefined, on ne les modifie pas
        if (nom_produit_new !== undefined) {
            sql += `nom_produit = '${nom_produit_new}', `;
        }
        if (date_peremption_new !== undefined) {
            sql += `date_peremption = '${date_peremption_new}', `;
        }
        if (quantite_stock !== undefined) {
            sql += `quantite_stock = ${quantite_stock}, `;
        }

        // note : le slice permet de supprimer la virgule en trop
        sql = sql.slice(0, -2) + ` WHERE nom_produit = '${nom_produit_old}' AND date_peremption = '${date_peremption_old}';`;
        console.log("SQL:", sql);

        return run(sql);
    },


    // DELETE
    delete: (nom_produit, date_peremption) =>
        run(`DELETE FROM STOCK WHERE nom_produit = '${nom_produit}' AND date_peremption = '${date_peremption}';`),
};

